/**
 * Redis-session类, 存入redis key-val
 * @authors zhanglianmin (28563760@qq.com)
 * @date    2017-08-21 16:08:57
 *
 */

"use strict";

let ioredis = require('ioredis')

class FsWebRedisSession {

    constructor(conf){
        
        this.redis = new ioredis({
            host: conf.conf.host || '127.0.0.1',
            port: conf.conf.port || 6379,
            db: conf.conf.db || 0
        })

        this.ttl = conf.ttl
    }

    init(clientID){
        this.clientID = clientID; //一般useAgent + IP
    }


    createSsid(ssid){
        let uuid = libs.SEC.md5(this.clientID)
        if(!ssid || ssid.slice(10) !== uuid){
            ssid = Math.random().toString(16).slice(5) + uuid
        }
        this.ssid = ssid
        //设置session有效期
        this.redis.expire(this.ssid, this.ttl)
    }


    start(ssid){
        this.createSsid(ssid)
        return this.ssid
    }


    //获取session字段值, 需要yeild指令
    get(key){
        return new Promise((yes, no) => {
            this.redis.hgetall(this.ssid, (err, obj) => {
                if(err)
                    return no(err)

                for(let i in obj){
                    if(isFinite(obj[i]))
                        obj[i] -= 0
                }
                //不传key时,直接返回全部字段
                if(!key)
                    yes(obj)
                else
                    yes(obj.hasOwnProperty(key) ? obj[key] : null)
            })
        })
    }


    //设置session字段值
    set(key, val){
        if(typeof key === 'object'){
            for(let i in key){
                this.redis.hset(this.ssid, i, key[i], (err, code)=>{
                    if(!err)
                        this.redis.expire(this.ssid, this.ttl)
                })
            }
        }else{
            this.redis.hset(this.ssid, key, val, (err, code)=>{
                if(!err)
                    this.redis.expire(this.ssid, this.ttl)
            })
        }
    }


    //删除单个字段
    unset(key){
        this.redis.hdel(this.ssid, key)
    }

    //清除个人session
    clear(){
        this.redis.del(this.ssid)
    }

}


module.exports = FsWebRedisSession